#define TESTAPP_GEN

/* $Id: */
/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2005-2006 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/*****************************************************************************/
/**
*
* @file xtemac_example_polled.c
*
* Implements examples that utilize the TEMAC's FIFO direct frame transfer
* mode in a polled fashion to send and receive frames.
*
* These examples demonstrate:
*
* - How to perform simple polling send and receive.
* - Advanced frame processing
* - Error handling
*
* Functional guide to example:
*
* - TemacSingleFramePolledExample() demonstrates the simplest way to send and
*   receive frames in polled mode.
*
* - TemacMultipleFramesPolledExample() demonstrates how to transmit a "burst" of
*   frames by queueing up several in the packet FIFO prior to transmission.
*
* - TemacPollForTxStatus() demonstrates how to poll for transmit complete status
*   and how to handle error conditions.
*
* - TemacPollForRxStatus() demonstrates how to poll for receive status and how
*   to handle error conditions.
*
* - TemacResetDevice() demonstrates how to reset the driver/HW without losing
*   all configuration settings.
*
* Note that the advanced frame processing algorithms shown here are not limited
* to polled mode operation. The same techniques can be used for FIFO direct
* interrupt driven mode as well.
*
* <pre>
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -------------------------------------------------------
* 1.00a rmm  06/01/05 First release
* 2.00a rmm  11/21/05 Added call to TemaUtilEnterLoopback(),
* 2.00a sv   06/12/06 Minor changes to comply to Doxygen and coding guidelines
*
* </pre>
*
******************************************************************************/

/***************************** Include Files *********************************/

#include "xtemac_example.h"

/************************** Constant Definitions ****************************/

/*
 * The following constants map to the XPAR parameters created in the
 * xparameters.h file. They are defined here such that a user can easily
 * change all the needed parameters in one place.
 */
#ifndef TESTAPP_GEN
#define TEMAC_DEVICE_ID   XPAR_PLB_TEMAC_0_DEVICE_ID
#endif

/************************** Variable Definitions ****************************/

EthernetFrame TxFrame;    /* Transmit frame buffer */
EthernetFrame RxFrame;    /* Receive frame buffer */

/************************** Function Prototypes *****************************/

XStatus TemacPolledExample(Xuint16 TemacDeviceId);
XStatus TemacSingleFramePolledExample();
XStatus TemacMultipleFramesPolledExample();

XStatus TemacPollForTxStatus();
XStatus TemacPollForRxStatus();
XStatus TemacResetDevice();

/*****************************************************************************/
/**
*
* This is the main function for the Temac example. This function is not included
* if the example is generated from the TestAppGen test  tool.
*
* @param    None.
*
* @return   XST_SUCCESS to indicate success, otherwise XST_FAILURE.
*
* @note     None.
*
****************************************************************************/
#ifndef TESTAPP_GEN
int main(void)
{
    XStatus Status;


    /*
     * Call the Temac polled example , specify the Device ID generated in
     * xparameters.h
     */
    Status = TemacPolledExample(TEMAC_DEVICE_ID);
    if (Status != XST_SUCCESS)
    {
        return XST_FAILURE;
    }

    return XST_SUCCESS;

}
#endif

/*****************************************************************************/
/**
*
* This function demonstrates the usage of the TEMAC by sending and receiving
* frames in polled mode.
*
*
* @param    TemacDeviceId is device ID of the Temac Device , typically
*           XPAR_<TEMAC_instance>_DEVICE_ID value from xparameters.h
*
* @return   XST_SUCCESS to indicate success, otherwise XST_FAILURE
*
* @note     None.
*
******************************************************************************/
XStatus TemacPolledExample(Xuint16 TemacDeviceId)
{
    XStatus Status;

    /*************************************/
    /* Setup device for first-time usage */
    /*************************************/

    /*
     * Initialize the TEMAC instance
     */
    Status = XTemac_Initialize(&TemacInstance, TemacDeviceId);
    if (Status != XST_SUCCESS)
    {
        TemacUtilErrorTrap("Error in initialize");
        return XST_FAILURE;
    }

    /*
     * Check whether the IPIF interface is correct for this example
     */
    if (!XTemac_mIsFifo(&TemacInstance))
    {
        TemacUtilErrorTrap("Device HW not configured for FIFO direct mode\r\n");
        return XST_FAILURE;
    }

    /*
     * Set the MAC  address
     */
    Status = XTemac_SetMacAddress(&TemacInstance, (Xuint8*)TemacMAC);
    if (Status != XST_SUCCESS)
    {
        TemacUtilErrorTrap("Error setting MAC address");
        return XST_FAILURE;
    }

    /*
     * Enable options needed for all polled examples
     */
    Status = XTemac_SetOptions(&TemacInstance, XTE_POLLED_OPTION);
    if (Status != XST_SUCCESS)
    {
        TemacUtilErrorTrap("Error setting options");
        return XST_FAILURE;
    }

    /*
     * Set PHY to loopback
     */
    usleep(10000);              /* Delay after setting the MAC address */
    Status = TemacUtilEnterLoopback(&TemacInstance, TEMAC_LOOPBACK_SPEED);
    if (Status != XST_SUCCESS)
    {
        TemacUtilErrorTrap("Error setting the PHY loopback");
        return XST_FAILURE;
    }


    /*
     * Set PHY<-->MAC data clock
     */
    XTemac_SetOperatingSpeed(&TemacInstance, TEMAC_LOOPBACK_SPEED);


    /****************************/
    /* Run through the examples */
    /****************************/

    /*
     * Run the Single Frame polled example
     */
    Status = TemacSingleFramePolledExample();
    if (Status != XST_SUCCESS)
    {
        return XST_FAILURE;
    }


#ifndef TESTAPP_GEN
    /*
     * Run the Multiple Frames polled example
     */
    Status = TemacMultipleFramesPolledExample();
    if (Status != XST_SUCCESS)
    {
        return XST_FAILURE;
    }
#endif

    return XST_SUCCESS;


}


/*****************************************************************************/
/**
*
* This function demonstrates the usage of the TEMAC by sending and receiving
* a single frame in polled mode.
*
* @param    None.
*
* @return   XST_SUCCESS to indicate success, otherwise XST_FAILURE.
*
* @note     None.
*
******************************************************************************/
XStatus TemacSingleFramePolledExample(void)
{
    XStatus Status;
    Xuint32 FifoFreeBytes;
    int PayloadSize = 100;
    Xuint32 TxFrameLength;
    Xuint32 RxFrameLength;

    /*
     * Start the TEMAC device
     */
    Status = XTemac_Start(&TemacInstance);
    if (Status != XST_SUCCESS)
    {
        TemacUtilErrorTrap("Error starting devicet");
        return XST_FAILURE;
    }

    /*
     * Setup the packet to be transmitted
     */
    TemacUtilFrameHdrFormatMAC(&TxFrame, TemacMAC);
    TemacUtilFrameHdrFormatType(&TxFrame, PayloadSize);
    TemacUtilFrameSetPayloadData(&TxFrame, PayloadSize);

    /*
     * Clear out the receive packet memory area
     */
    TemacUtilFrameMemClear(&RxFrame);

    /*
     * Calculate frame length (not including FCS)
     */
    TxFrameLength = XTE_HDR_SIZE + PayloadSize;

    /*******************/
    /* Send the packet */
    /*******************/

    /*
     * Wait for enough room in FIFO to become available
     */
    do
    {
        FifoFreeBytes = XTemac_FifoGetFreeBytes(&TemacInstance, XTE_SEND);
    } while (FifoFreeBytes < TxFrameLength);

    /*
     * Write the frame data to FIFO
     */
    Status = XTemac_FifoWrite(&TemacInstance,
                              TxFrame,
                              TxFrameLength,
                              XTE_END_OF_PACKET);
    if (Status != XST_SUCCESS)
    {
        TemacUtilErrorTrap("Error writing to FIFO");
        TemacResetDevice();
        return XST_FAILURE;
    }

    /*
     * Initiate transmit
     */
    Status = XTemac_FifoSend(&TemacInstance, TxFrameLength);
    if (Status != XST_SUCCESS)
    {
        TemacUtilErrorTrap("Error on transmit");
        TemacResetDevice();
        return XST_FAILURE;
    }

    /*
     * Wait for status of the transmitted packet
     */
    switch(TemacPollForTxStatus())
    {
        case XST_SUCCESS:  /* Got a sucessfull transmit status */
            break;

        case XST_NO_DATA:  /* Timed out */
            TemacUtilErrorTrap("Tx timeout");
            return XST_FAILURE;

        default:           /* Some other error */
            return XST_FAILURE;
    }

    /**********************/
    /* Receive the packet */
    /**********************/

    /*
     * Wait for packet Rx
     */
    switch(TemacPollForRxStatus())
    {
        case XST_SUCCESS:  /* Got a sucessfull receive status */
            break;

        case XST_NO_DATA:  /* Timed out */
            TemacUtilErrorTrap("Rx timeout");
            return XST_FAILURE;

        default:           /* Some other error */
            return XST_FAILURE;
    }

    /*
     * A packet as arrived, get its length
     */
    Status = XTemac_FifoRecv(&TemacInstance, &RxFrameLength);
    if (Status != XST_SUCCESS)
    {
        TemacUtilErrorTrap("Packet has arrived, but there is no length");
        return XST_FAILURE;
    }

    /*
     * Read the received packet data
     */
    Status = XTemac_FifoRead(&TemacInstance,
                             &RxFrame,
                             RxFrameLength,
                             XTE_END_OF_PACKET);
    if (Status != XST_SUCCESS)
    {
        TemacUtilErrorTrap("Error in FifoRead");
        TemacResetDevice();
        return XST_FAILURE;
    }

    /*
     * Verify the received frame length
     */
    if ((RxFrameLength) != TxFrameLength)
    {
        TemacUtilErrorTrap("Receive length incorrect");
        return XST_FAILURE;
    }

    /*
     * Validate frame data
     */
    if (TemacUtilFrameVerify(&TxFrame, &RxFrame) != 0)
    {
        TemacUtilErrorTrap("Receive Data mismatch");
        return XST_FAILURE;
    }

    /*
     * Stop device
     */
    XTemac_Stop(&TemacInstance);

    return XST_SUCCESS;
}

/*****************************************************************************/
/**
*
* This example uses polled mode to queue up multiple frames in the packet
* FIFOs before sending them in a single burst. Receive packets are handled in
* a similar way.
*
* @param    None.
*
* @return   XST_SUCCESS to indicate success, otherwise XST_FAILURE.
*
* @note     None.
*
******************************************************************************/
XStatus TemacMultipleFramesPolledExample(void)
{
    Xuint32 FramesToLoopback;
    Xuint32 PayloadSize;
    XStatus Status;
    Xuint32 TxFrameLength;
    Xuint32 RxFrameLength;
    Xuint32 Index;

    /*
     * Start the TEMAC device
     */
    Status = XTemac_Start(&TemacInstance);
    if (Status != XST_SUCCESS)
    {
        TemacUtilErrorTrap("Error starting devicet");
        return XST_FAILURE;
    }

    /*
     * Setup the number of frames to loopback (FramesToLoopback) and the size
     * of the frame (PayloadSize) to loopback. The default settings should
     * work for every case. Modifying the settings can cause problems, see
     * discussion below:
     *
     * If PayloadSize is set small and FramesToLoopback high, then it is
     * possible to cause the transmit status FIFO to overflow.
     *
     * If PayloadSize is set large and FramesToLoopback high, then it is
     * possible to cause the transmit packet FIFO to overflow.
     *
     * Either of these scenarios may be worth trying out to observe how the
     * driver reacts. The exact values to cause these types of errors
     * will vary due to the sizes of the FIFOs selected at hardware build
     * time. But the following settings should create problems for all
     * FIFO sizes:
     *
     * Transmit status FIFO overflow
     *    PayloadSize = 1
     *    FramesToLoopback = 1000
     *
     * Transmit packet FIFO overflow
     *    PayloadSize = 1500
     *    FramesToLoopback = 16
     *
     * These values should always work without error
     *    PayloadSize = 100
     *    FramesToLoopback = 5
     */
    PayloadSize = 100;
    FramesToLoopback = 5;

    /*
     * Calculate Tx frame length (not including FCS)
     */
    TxFrameLength = XTE_HDR_SIZE + PayloadSize;

    /*
     * Setup the packet to be transmitted
     */
    TemacUtilFrameHdrFormatMAC(&TxFrame, TemacMAC);
    TemacUtilFrameHdrFormatType(&TxFrame, PayloadSize);
    TemacUtilFrameSetPayloadData(&TxFrame, PayloadSize);

    /****************/
    /* Send packets */
    /****************/

    /*
     * Since we may be interested to see what happens when FIFOs overflow, don't
     * check for room in the transmit packet FIFO prior to writing to it.
     */

    /*
     * Write frame data to FIFO
     */
    for (Index=0; Index < FramesToLoopback; Index++)
    {
        Status = XTemac_FifoWrite(&TemacInstance,
                                  TxFrame,
                                  TxFrameLength,
                                  XTE_END_OF_PACKET);
        if (Status != XST_SUCCESS)
        {
            TemacUtilErrorTrap("Error writing to FIFO");
            TemacResetDevice();
            return XST_FAILURE;
        }
    }

    /*
     * Initiate the transmission
     */
    for (Index=0; Index < FramesToLoopback; Index++)
    {
        Status = XTemac_FifoSend(&TemacInstance, TxFrameLength);
        if (Status != XST_SUCCESS)
        {
            TemacUtilErrorTrap("Error on transmit");
            TemacResetDevice();
            return XST_FAILURE;
        }
    }

    /*
     * Wait for transmit result
     */
    for (Index=0; Index < FramesToLoopback; Index++)
    {
        switch(TemacPollForTxStatus())
        {
            case XST_SUCCESS:  /* Got a sucessfull transmit status */
                break;

            case XST_NO_DATA:  /* Timed out */
                TemacUtilErrorTrap("Tx timeout");
                return XST_FAILURE;
                break;

            default:           /* Some other error */
                TemacResetDevice();
                return XST_FAILURE;
        }
    }

    /**********************/
    /* Receive the packet */
    /**********************/

    /*
     * Wait for the packets to arrive
     */
    for (Index=0; Index<FramesToLoopback; Index++)
    {
        /*
         * Wait for packet Rx
         */
        switch(TemacPollForRxStatus())
        {
            case XST_SUCCESS:  /* Got a successfull receive status */
                break;

            case XST_NO_DATA:  /* Timed out */
                TemacUtilErrorTrap("Rx timeout");
                return XST_FAILURE;
                break;

            default:           /* Some other error */
                TemacResetDevice();
                return XST_FAILURE;
        }

        /*
         * A packet has arrived, get its length
         */
        Status = XTemac_FifoRecv(&TemacInstance, &RxFrameLength);
        if (Status != XST_SUCCESS)
        {
            TemacUtilErrorTrap("Packet has arrived, but there is no length");
            TemacResetDevice();
            return XST_FAILURE;
        }

        /*
         * Verify the received frame length
         */
        if ((RxFrameLength) != TxFrameLength)
        {
            TemacUtilErrorTrap("Receive length incorrect");
            return XST_FAILURE;
        }
    }

    /*
     * Read the data out of the receive packet FIFO and validate
     */
    for (Index=0; Index<FramesToLoopback; Index++)
    {
        /*
         * Clear out receive packet memory area
         */
        TemacUtilFrameMemClear(&RxFrame);

        /*
         * Transfer out of FIFO
         */
        Status = XTemac_FifoRead(&TemacInstance,
                                 &RxFrame,
                                 RxFrameLength,
                                 XTE_END_OF_PACKET);
        if (Status != XST_SUCCESS)
        {
            TemacUtilErrorTrap("Error in FifoRead");
            TemacResetDevice();
            return XST_FAILURE;
        }

        if (TemacUtilFrameVerify(&TxFrame, &RxFrame) != 0)
        {
            TemacUtilErrorTrap("Receive Data Mismatch");
            return XST_FAILURE;
        }
    }

    /*
     * Stop device
     */
    XTemac_Stop(&TemacInstance);

    return XST_SUCCESS;
}


/******************************************************************************/
/**
* This functions polls the Tx status and waits for an indication that a frame
* has been transmitted successfully or a transmit related error has occurred.
* If an error is reported, it handles all the possible  error conditions.
*
* @param    None.
*
* @return   Status is the status of the last call to the
*           XTemac_FifoQuerySendStatus() function.
*
* @note     None.
*
******************************************************************************/
XStatus TemacPollForTxStatus(void)
{
    Xuint32 SendStatus;
    XStatus Status = XST_NO_DATA;
    int Attempts = 100000;  /* Number of attempts to get status before giving
                               up */

    /*
     * Wait for transmit complete indication
     */
    do
    {

        if (--Attempts <= 0) break;         /* Give up? */

        /*
         * Try to get the status
         */
        Status = XTemac_FifoQuerySendStatus(&TemacInstance, &SendStatus);

    } while(Status == XST_NO_DATA);

    switch(Status)
    {
        case XST_SUCCESS:  /* Frame sent without error */
        case XST_NO_DATA:  /* Timeout */
            break;

        case XST_PFIFO_ERROR:
            TemacUtilErrorTrap("Packet FIFO error");
            TemacResetDevice();
            break;

        case XST_IPIF_ERROR:
            TemacUtilErrorTrap("IPIF error");
            TemacResetDevice();
            break;

        case XST_FIFO_ERROR:
            TemacUtilErrorTrap("Status/Length FIFO error");
            TemacResetDevice();
            break;

        case XST_NOT_POLLED:
            TemacUtilErrorTrap("Driver was not in polled mode");
            break;

        case XST_DEVICE_IS_STOPPED:
            TemacUtilErrorTrap("Device was stopped prior to polling operation");
            break;

        default:
            TemacUtilErrorTrap("Driver returned unknown transmit status");
            break;
    }

    return(Status);
}


/******************************************************************************/
/**
* This functions polls the Rx status and waits for an indication that a frame
* has arrived or a receive related error has occurred. If an error is reported,
* handle all the possible  error conditions.
*
* @param    None.
*
* @return   Status is the status of the last call to the
*           XTemac_FifoQueryRecvStatus() function.
*
* @note     None.
*
******************************************************************************/
XStatus TemacPollForRxStatus(void)
{
    XStatus Status = XST_NO_DATA;
    int Attempts = 10000000;  /* number of times to get a status before
                                 giving up */

    /*
     * There are two ways to poll for a received frame:
     *
     * XTemac_Recv() can be used and repeatedly called until it returns a
     * length,  but this method does not provide any error detection.
     *
     * XTemac_FifoQueryRecvStatus() can be used and this function provides
     * more information to handle error conditions.
     */

    /*
     * Wait for something to happen
     */
    do
    {
        if (--Attempts <= 0) break; /* Give up? */

        /*
         * Try to get status
         */
        Status = XTemac_FifoQueryRecvStatus(&TemacInstance);

    } while(Status == XST_NO_DATA);

    switch(Status)
    {
        case XST_SUCCESS:  /* Frame has arrived */
        case XST_NO_DATA:  /* Timeout */
            break;

        case XST_DATA_LOST:
            TemacUtilErrorTrap("Frame was dropped");
            break;

        case XST_PFIFO_ERROR:
            TemacUtilErrorTrap("Packet FIFO error");
            TemacResetDevice();
            break;

        case XST_IPIF_ERROR:
            TemacUtilErrorTrap("IPIF error");
            TemacResetDevice();
            break;

        case XST_FIFO_ERROR:
            TemacUtilErrorTrap("Length/status FIFO error");
            TemacResetDevice();
            break;

        case XST_DEVICE_IS_STOPPED:
            TemacUtilErrorTrap("Device was stopped prior to polling operation");
            break;

        default:
            TemacUtilErrorTrap("Driver returned invalid transmit status");
            break;
    }

    return(Status);
}


/******************************************************************************/
/**
* This function resets the device but preserves the options set by the user.
*
* @param    None.
*
* @return   XST_SUCCESS if successful, else XST_FAILURE.
*
* @note     None.
*
******************************************************************************/
XStatus TemacResetDevice(void)
{
    XStatus Status;
    Xuint8 MacSave[6];
    Xuint32 Options;

    /*
     * Stop device
     */
    XTemac_Stop(&TemacInstance);

    /*
     * Save the device state
     */
    XTemac_GetMacAddress(&TemacInstance, MacSave);
    Options = XTemac_GetOptions(&TemacInstance);

    /*
     * Stop and reset the device
     */
    XTemac_Reset(&TemacInstance, XTE_NORESET_HARD);

    /*
     * Restore the state
     */
    Status  = XTemac_SetMacAddress(&TemacInstance, MacSave);
    Status |= XTemac_SetOptions(&TemacInstance, Options);
    Status |= XTemac_ClearOptions(&TemacInstance, ~Options);
    if (Status != XST_SUCCESS)
    {
        TemacUtilErrorTrap("Error restoring state after reset");
        return XST_FAILURE;
    }

    return XST_SUCCESS;
}




