#define TESTAPP_GEN

/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2005-2006 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/*****************************************************************************/
/**
*
* @file xtemac_example_util.c
*
* This file implements the utility functions for the Temac example code.
*
* <pre>
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -------------------------------------------------------
* 1.00a rmm  06/01/05 First release
* 2.00a rmm  11/21/05 Added Util_EnterLoopback() & Util_SetupUart()
* 2.00a sv   06/12/06 Minor changes to comply to Doxygen and coding guidelines
* 2.00a rpm  09/14/06 Added extra usleep between PhyRead and PhyWrite
* 2.00a rpm  10/30/06 Added code to detect the PHY address
* </pre>
*
******************************************************************************/

/***************************** Include Files *********************************/

#include "xtemac_example.h"



/************************** Variable Definitions ****************************/

XTemac TemacInstance;   /* XTemac instance used throughout examples */

/*
 * Local MAC address
 */
char TemacMAC[6] = { 0x06, 0x05, 0x04, 0x03, 0x02, 0x01 };


/******************************************************************************/
/**
*
* Set the MAC addresses in the frame.
*
* @param    FramePtr is the pointer to the frame.
* @param    DestAddr is the Destination MAC address.
*
* @return   None.
*
* @note     None.
*
******************************************************************************/
void TemacUtilFrameHdrFormatMAC(EthernetFrame *FramePtr, char* DestAddr)
{
    char* Frame = (char*)FramePtr;
    char* SourceAddress = TemacMAC;
    int Index;

    /* destination address */
    for (Index=0; Index<XTE_MAC_ADDR_SIZE; Index++)
    {
        *Frame++ = *DestAddr++;
    }

    /* source address */
    for (Index=0; Index < XTE_MAC_ADDR_SIZE; Index++)
    {
        *Frame++ = *SourceAddress++;
    }
}

/******************************************************************************/
/**
*
* Set the frame type for the specified frame.
*
* @param    FramePtr is the pointer to the frame.
* @param    FrameType is the Type to set in frame.
*
* @return   None.
*
* @note     None.
*
******************************************************************************/
void TemacUtilFrameHdrFormatType(EthernetFrame *FramePtr, Xuint16 FrameType)
{
    char* Frame = (char*)FramePtr;

    /*
     * Increment to type field
     */
    Frame = Frame + 12;

    /*
     * Set the type
     */
    *(Xuint16*)Frame = FrameType;
}

/******************************************************************************/
/**
* This function places a pattern in the payload section of a frame. The pattern
* is a  8 bit incrementing series of numbers starting with 0.
* Once the pattern reaches 256, then the pattern changes to a 16 bit
* incrementing pattern:
* <pre>
*   0, 1, 2, ... 254, 255, 00, 00, 00, 01, 00, 02, ...
* </pre>
*
* @param    FramePtr is a pointer to the frame to change.
* @param    PayloadSize is the number of bytes in the payload that will be set.
*
* @return   None.
*
* @note     None.
*
******************************************************************************/
void TemacUtilFrameSetPayloadData(EthernetFrame *FramePtr, int PayloadSize)
{
    unsigned BytesLeft = PayloadSize;
    Xuint8* Frame;
    Xuint16 Counter = 0;

    /*
     * Set the frame pointer to the start of the payload area
     */
    Frame = (Xuint8*)FramePtr + XTE_HDR_SIZE;

    /*
     * Insert 8 bit incrementing pattern
     */
    while(BytesLeft && (Counter < 256))
    {
        *Frame++ = (Xuint8)Counter++;
        BytesLeft--;
    }

    /*
     * Switch to 16 bit incrementing pattern
     */
    while(BytesLeft)
    {
        *Frame++ = (Xuint8)(Counter >> 8);  /* high */
        BytesLeft--;

        if (!BytesLeft) break;

        *Frame++ = (Xuint8)Counter++;        /* low */
        BytesLeft--;
    }
}

/******************************************************************************/
/**
* This function verifies the frame data against a CheckFrame.
*
* Validation occurs by comparing the ActualFrame to the header of the
* CheckFrame. If the headers match, then the payload of ActualFrame is
* verified for the same pattern Util_FrameSetPayloadData() generates.
*
* @param    CheckFrame is a pointer to a frame containing the 14 byte header
*           that should be present in the ActualFrame parameter.
* @param    ActualFrame is a pointer to a frame to validate.
*
* @return   XST_SUCCESS if successful, else XST_FAILURE.
*
* @note     None.
******************************************************************************/
int TemacUtilFrameVerify(EthernetFrame *CheckFrame, EthernetFrame *ActualFrame)
{
    char* CheckPtr = (char*)CheckFrame;
    char* ActualPtr = (char*)ActualFrame;
    Xuint16 BytesLeft;
    Xuint16 Counter;
    int Index;

    /*
     * Compare the headers
     */
    for (Index = 0; Index < XTE_HDR_SIZE; Index++)
    {
        if (CheckPtr[Index] != ActualPtr[Index])
        {
            return XST_FAILURE;
        }
    }

    /*
     * Get the length of the payload
     */
    BytesLeft = *(Xuint16*)&ActualPtr[12];

    /*
     * Validate the payload
     */
    Counter = 0;
    ActualPtr = &ActualPtr[14];

    /*
     * Check 8 bit incrementing pattern
     */
    while(BytesLeft && (Counter < 256))
    {
        if (*ActualPtr++ != (Xuint8)Counter++)
        {
            return XST_FAILURE;
        }
        BytesLeft--;
    }

    /*
     * Check 16 bit incrementing pattern
     */
    while(BytesLeft)
    {
        if (*ActualPtr++ != (Xuint8)(Counter >> 8))  /* high */
        {
            return XST_FAILURE;
        }

        BytesLeft--;

        if (!BytesLeft) break;

        if (*ActualPtr++ != (Xuint8)Counter++)       /* low */
        {
            return XST_FAILURE;
        }

        BytesLeft--;
    }

    return XST_SUCCESS;
}

/******************************************************************************/
/**
* This function sets all bytes of a frame to 0.
*
* @param    FramePtr is a pointer to the frame itself.
*
* @return   None.
*
* @note     None.
*
******************************************************************************/
void TemacUtilFrameMemClear(EthernetFrame *FramePtr)
{
    Xuint32 *Data32Ptr = (Xuint32*)FramePtr;
    Xuint32 WordsLeft = sizeof(EthernetFrame)/sizeof(Xuint32);

    /* frame should be an integral number of words */
    while(WordsLeft--)
    {
        *Data32Ptr++ = 0;
    }
}

/******************************************************************************/
/**
*
* This function detects the PHY address by looking for successful MII status
* register contents (PHY register 1). It looks for a PHY that supports
* auto-negotiation and 10Mbps full-duplex and half-duplex.  So, this code
* won't work for PHYs that don't support those features, but it's a bit more
* general purpose than matching a specific PHY manufacturer ID.
*
* Note also that on some (older) Xilinx ML4xx boards, PHY address 0 does not
* properly respond to this query.  But, since the default is 0 and asssuming
* no other address responds, then it seems to work OK.
*
* @param    The TEMAC driver instance
*
* @return   The address of the PHY (defaults to 0 if none detected)
*
* @note     None.
*
******************************************************************************/
/* Use MII register 1 (MII status register) to detect PHY */
#define PHY_DETECT_REG  1

/* Mask used to verify certain PHY features (or register contents)
 * in the register above:
 *  0x1000: 10Mbps full duplex support
 *  0x0800: 10Mbps half duplex support
 *  0x0008: Auto-negotiation support
 */
#define PHY_DETECT_MASK 0x1808

Xuint32 TemacDetectPHY(XTemac *TemacInstancePtr)
{
    XStatus Status;
    Xuint16 PhyReg;
    Xuint32 PhyAddr;

    for (PhyAddr = 0; PhyAddr <= 31; PhyAddr++)
    {
        Status = XTemac_PhyRead(TemacInstancePtr, PhyAddr, PHY_DETECT_REG,
                                &PhyReg);

        if ((Status == XST_SUCCESS) && (PhyReg != 0xFFFF) &&
            (PhyReg & PHY_DETECT_MASK == PHY_DETECT_MASK))
        {
            /* Found a valid PHY address */
            return PhyAddr;
        }
    }

    return 0;   /* default to zero */
}


/******************************************************************************/
/**
 * Set PHY to loopback mode. This works with the marvell PHY common on ML40x
 * evaluation boards
 *
 * @param Speed is the loopback speed 10, 100, or 1000 Mbit
 *
 ******************************************************************************/
#define PHY_R0_RESET         0x8000
#define PHY_R0_LOOPBACK      0x4000
#define PHY_R20_DFT_SPD_MASK 0x0070    /* Register 20 is PHY specific */
#define PHY_R20_DFT_SPD_10   0x0040
#define PHY_R20_DFT_SPD_100  0x0050
#define PHY_R20_DFT_SPD_1000 0x0060


/******************************************************************************/
/**
*
* This function sets the PHY to loopback mode. This works with the marvell PHY
* common on ML40x evaluation boards.
*
* @param    Speed is the loopback speed 10, 100, or 1000 Mbit.
*
* @return   XST_SUCCESS if successful, else XST_FAILURE.
*
* @note     None.
*
******************************************************************************/
XStatus TemacUtilEnterLoopback(XTemac *TemacInstancePtr, int Speed)
{
    XStatus Status;
    Xuint16 PhyReg0;
    Xuint16 PhyReg20;
    Xuint32 PhyAddr;

    /* Detec the PHY address */
    PhyAddr = TemacDetectPHY(TemacInstancePtr);

    Status = XTemac_PhyRead(TemacInstancePtr, PhyAddr, 0, &PhyReg0);
    Status |= XTemac_PhyRead(TemacInstancePtr, PhyAddr, 20, &PhyReg20);
    if (Status != XST_SUCCESS)
    {
        TemacUtilErrorTrap("Error reading phy");
        return XST_FAILURE;
    }

    PhyReg20 &= ~PHY_R20_DFT_SPD_MASK;

    switch(Speed)
    {
        case 10:
            PhyReg20 |= PHY_R20_DFT_SPD_10;
            break;

        case 100:
            PhyReg20 |= PHY_R20_DFT_SPD_100;
            break;

        case 1000:
            PhyReg20 |= PHY_R20_DFT_SPD_1000;
            break;

        default:
            TemacUtilErrorTrap("Intg_LinkSpeed not 10, 100, or 1000 mbps");
            return XST_FAILURE;
    }

    /* There seem to be some sleeps needing between PhyReads/PhyWrites */
    usleep(1000000);
    Status = XTemac_PhyWrite(TemacInstancePtr, PhyAddr, 20, PhyReg20);
    Status |= XTemac_PhyWrite(TemacInstancePtr, PhyAddr, 0,
                              PhyReg0 | PHY_R0_RESET);
    usleep(1000000 * TEMAC_PHY_DELAY_SEC);
    Status |= XTemac_PhyWrite(TemacInstancePtr, PhyAddr, 0,
                              PhyReg0 | PHY_R0_LOOPBACK);
    usleep(1000000);


    if (Status != XST_SUCCESS)
    {
        TemacUtilErrorTrap("Error writing phy");
        return XST_FAILURE;
    }

    return XST_SUCCESS;
}

/******************************************************************************/
/**
*
* This function is called by example code when an error is detected. It
* can be set as a breakpoint with a debugger or it can be used to print out the
* given message if there is a UART or STDIO device.
*
* @param    Message is the text explaining the error
*
* @return   None
*
* @note     None
*
******************************************************************************/
void TemacUtilErrorTrap(char* Message)
{
    static int Count = 0;

    Count++;

#ifdef STDOUT_BASEADDRESS
    xil_printf("%s\r\n", Message);
#endif
}

